/*
 *  Copyright (c) Members of the EGEE Collaboration. 2005.
 *  See http://eu-egee.org/partners/ for details on the copyright holders.
 *  For license conditions see the license file or http://eu-egee.org/license.html
 *
 *  GLite Data Management - Simple Fireman API
 *
 *  Authors: 
 *      Gabor Gombas <Gabor.Gombas@cern.ch>
 *      Akos Frohner <Akos.Frohner@cern.c>
 *  Version info: $Id: fireman-simple.h,v 1.24 2005/04/25 13:25:21 dalion Exp $
 *  Release: $Name: glite-data-catalog-cli_R_1_7_2_6 $
 *
 */

#ifndef GLITE_DATA_CATALOG_FIREMAN_SIMPLE_H
#define GLITE_DATA_CATALOG_FIREMAN_SIMPLE_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdarg.h>
#include <stdint.h>
#include <time.h>

#include <glite/data/catalog/c/catalog-simple.h>

/**
 * Callback function for path expansion
 */
typedef int (*glite_fireman_exp_cb)(glite_catalog_ctx *ctx,
		glite_catalog_FCEntry *entry,
		void *user_ptr,
		glite_catalog_exp_dir direction);

/**
 * \defgroup catalog_highlevel Catalog Utilities
 * @{
 */

/**
 * Expand (possibly recursively) a path containing glob meta characters and
 * call the callback function on every expanded entry. 
 *
 * @param ctx The general catalog-context. 
 * @param path The path - possibly with wildcards, that should be walked through.
 * @param cb The function that will be called for every matching LFN.
 * @param flags Various flags of the operation.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_expand_path(glite_catalog_ctx *ctx, const char *path,
	glite_fireman_exp_cb cb, void *user_ptr, glite_catalog_exp_flag flags);

/**
 * Get the largest limit value that can be passed to readDir(), or 0
 * if the limit is not known. 
 *
 * @param ctx The general catalog-context. 
 *
 * @return The limit value of a readDir(), or 0. @see readDir()
 */
int glite_fireman_get_readDir_limit(glite_catalog_ctx *ctx);

/**
 * Get the largest limit value that can be passed to locate(), or 0
 * if the limit is not known. 
 *
 * @param ctx The general catalog-context. 
 *
 * @return The limit value of a locate(), or 0. @see locate()
 */
int glite_fireman_get_locate_limit(glite_catalog_ctx *ctx);

/**
 * Get the largest limit value that can be passed to query(), or 0
 * if the limit is not known. 
 *
 * @param ctx The general catalog-context. 
 *
 * @return The limit value of a query(), or 0. @see query()
 */
int glite_fireman_get_query_limit(glite_catalog_ctx *ctx);

/**
 * @}
 */

/**
 * \defgroup catalog_service_ServiceBase org.glite.data.catalog.service.ServiceBase
 * @{
 */

/**
 * \brief Get the version of the remote endpoint.
 *
 * The returned string must be freed by the caller.
 *
 * @param ctx The general catalog-context. 
 *
 * @return The catalog version string, or NULL, if there was an error.
 */
char *glite_fireman_getVersion(glite_catalog_ctx *ctx);

/**
 * \brief Get the schema version of the remote endpoint.
 *
 * The returned string must be freed by the caller. 
 *
 * @param ctx The general catalog-context. 
 *
 * @return The schema version string, or NULL, if there was an error.
 */
char *glite_fireman_getSchemaVersion(glite_catalog_ctx *ctx);

/**
 * \brief Get the interface version of the remote endpoint.
 *
 * The returned string must be freed by the caller.
 *
 * @param ctx The general catalog-context. 
 *
 * @return The interface version string, or NULL, if there was an error.
 */
char *glite_fireman_getInterfaceVersion(glite_catalog_ctx *ctx);

/**
 * \brief Get the service metadata for a given key.
 *
 * The key must not be NULL.
 * The returned string must be freed by the caller.
 *
 * @param ctx The general catalog-context. 
 * @param key The key that's metadata we are looking for.
 *
 * @return The metadata string, or NULL, if there was an error.
 */
char *glite_fireman_getServiceMetadata(glite_catalog_ctx *ctx, const char *key);

/**
 * @}
 */

/**
 * \defgroup catalog_service_FASBase org.glite.data.catalog.service.FASBase
 * @{
 */

/**
 * Set the permissions for an item. 
 *
 * @param ctx The general catalog-context. 
 * @param item The string describing the item on which we set the permission.
 * @param permission The permission object.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_setPermission(glite_catalog_ctx *ctx, const char *item,
	const glite_catalog_Permission *permission);

/**
 * \brief Set the permissions for multiple items simultaneously. 
 *
 * The first item in the array gets the first permission object, 
 * the second one gets the second permission, and so on. 
 *
 * @param ctx The general catalog-context. 
 * @param nitems Number of items in the array.
 * @param items Array of items we want to set the permissions.
 * @param permissions Array of permission objects.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_setPermission_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const items[],
	const glite_catalog_Permission * const permissions[]);

/**
 * Get the permissions of an item. 
 *
 * @param ctx The general catalog-context. 
 * @param item A string describing the item that's permissions we want to get.
 *
 * @return The permission object of the item, or NULL, if there was an error.
 */
glite_catalog_Permission *glite_fireman_getPermission(glite_catalog_ctx *ctx,
	const char *item);

/**
 * \brief Get the permissions of multiple items.
 *
 * The result might contain NULL pointers if there 
 * were no response for that particular item.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param items Array of the items of which we want to get the permission.
 *
 * @return The array of permission objects. Some elements of the array may be NULL.
 */
glite_catalog_Permission **glite_fireman_getPermission_multi(glite_catalog_ctx *ctx,
	int nitems, const char * const items[]);

/**
 * \brief Check if we have all the requested permissions for an item.
 *
 * The permission bits can be combined using the logical or operation,
 * for example if you want read and write permissions, use: <BR>
 * ( ::GLITE_CATALOG_PERM_READ || ::GLITE_CATALOG_PERM_WRITE ).
 *
 * @param ctx The general catalog-context. 
 * @param item The item that's permissions we want to check. 
 * @param perm The permission bits we would need.
 *
 * @return 0, if we have sufficient permissions, and -1 in case of an error.
 *         If we do not have sufficient permissions, we get a
 *         ::GLITE_CATALOG_EXCEPTION_AUTHORIZATION error.
 */
int glite_fireman_checkPermission(glite_catalog_ctx *ctx, const char *item,
	glite_catalog_Perm perm);

/**
 * \brief Check if we have all the requested permissions for all items.
 *
 * Checks against the same permission bits on every items in the array.
 * The permission bits can be combined using the logical or operation,
 * for example if you want read and write permissions, use:<BR>
 * ( ::GLITE_CATALOG_PERM_READ || ::GLITE_CATALOG_PERM_WRITE ).
 *
 * @param ctx The general catalog-context. 
 * @param nitems Number of items in the array.
 * @param items Array of items.
 * @param perm The permission bits we would need.
 *
 * @return 0, if we have sufficient permissions, and -1 in case of an error.
 *         If we do not have ALL the requested permissions on EVERY items,
 *         we get a ::GLITE_CATALOG_EXCEPTION_AUTHORIZATION error.
 */
int glite_fireman_checkPermission_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const items[], glite_catalog_Perm perm);

/**
 * @}
 */

/**
 * \defgroup catalog_service_file_FileCatalog org.glite.data.catalog.service.file.FileCatalog
 * @{
 */

/**
 * Create a directory with the given flags. 
 *
 * @param ctx The general catalog-context. 
 * @param dir The Logical FileName of the directory.
 * @param flags The flags to be set.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_mkdir(glite_catalog_ctx *ctx, const char *dir,
	glite_catalog_DirOpFlag flags);

/**
 * Create a set of directories with the given flags. 
 *
 * @param ctx The general catalog-context. 
 * @param nitems Number of items in the array. 
 * @param dirs Array of directorie LFNs to be created.
 * @param flags The flags to be set.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_mkdir_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const dirs[], glite_catalog_DirOpFlag flags);

/**
 * Remove a directory. 
 *
 * @param ctx The general catalog-context. 
 * @param dir The LFN of the directory to be removed.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_rmdir(glite_catalog_ctx *ctx, const char *dir);

/**
 * Remove a set of directories. 
 *
 * @param ctx The general catalog-context. 
 * @param nitems Number of items in the array.
 * @param dirs Array of directories to be removed.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_rmdir_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const dirs[]);

/**
 * Create a file in the catalog.
 *
 * @param ctx  The general catalog-context.
 * @param entry The FileCatalog entry containing the informations about the file to be created.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_createFile(glite_catalog_ctx *ctx,
	const glite_catalog_FCEntry *entry);

/**
 * Create multiple files in the catalog.
 *
 * @param ctx  The general catalog-context.
 * @param nitems  Number of items in the array.
 * @param entries  Array of entries containing the informations about the files to be created.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_createFile_multi(glite_catalog_ctx *ctx, int nitems,
	const glite_catalog_FCEntry * const entries[]);

/**
 * \brief Create a symlink in the catalog.
 *
 * Creates an object with a given name that refers to the pointed object. Any file
 * operation (read, write, etc.) on the link will be translated to operations on the
 * pointed object. However, the link is based on the LFN, therefore if the name of the
 * pointed object changes, or the file gets removed, the link becomes invalid, but stays
 * in the catalog. The mechanism is similar to the UNIX-symlinks.
 * You may point a symlink on an existing symlink, which will cause the new symlink to point
 * to the object pointed by the existing symlink.
 *
 * @param ctx  The general catalog-context.
 * @param linkName Name of the link.
 * @param linkTarget Name of the object the link points to.
 * @param flags The flags of the operation.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_symlink(glite_catalog_ctx *ctx, const char *linkName,
	const char *linkTarget, glite_catalog_DirOpFlag flags);

/**
 * Bulk operation to create multiple symlinks. @see glite_fireman_symlink()
 *
 * @param ctx  The general catalog-context.
 * @param nitems Number of items in the array.
 * @param links Array of string-pairs. In every pair the first string is the name of the link,
 *              The second string is the name of the pointed object.
 * @param flags The flags of the operation.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_symlink_multi(glite_catalog_ctx *ctx, int nitems,
	const char *links[][2], glite_catalog_DirOpFlag flags);

/**
 * Delete an LFN, symlink or virtual directory. Use rmdir for normal directories.
 *
 * @param ctx The general catalog-context.
 * @param lfn The LFN to be disposed.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_unlink(glite_catalog_ctx *ctx, const char *lfn);

/**
 * Bulk operation to delete LFNs, symlinks or virtual directories.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param lfns Array of LFNs to be disposed.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_unlink_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const lfns[]);

/**
 * \brief Move a file or a directory.
 *
 * If the destination is a symlink, it will be resolved. If the
 * source is a symlink, the symlink will be moved.
 *
 * @param ctx The general catalog-context.
 * @param src The source filename.
 * @param dst The destination filename.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_mv(glite_catalog_ctx *ctx, const char *src,
	const char *dst);

/**
 * Bulk operation to move multiple files.
 * 
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param entries The array of string-pairs. The first one is the source,
 *                the second one is the destination. 
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_mv_multi(glite_catalog_ctx *ctx, int nitems,
	const char *entries[][2]);

/**
 * Update the modifyTime field of an LFN.
 * 
 * @param ctx The general catalog-context.
 * @param lfn The file that's modifyTime we want to update.
 * @param time The new value.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_updateModifyTime(glite_catalog_ctx *ctx, const char *lfn,
	const struct timespec *time);

/**
 * \brief Update the modifyTime field of many LFNs.
 *
 * The same time-value is set for each LFN.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param lfns Array of files we want to update.
 * @param time The new time-value.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_updateModifyTime_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const lfns[], const struct timespec *time);

/**
 * Updates the validityTime of a file in the catalog.
 *
 * @param ctx The general catalog-context.
 * @param lfn The file that's validityTime is to be updated.
 * @param time The new time-value.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_updateValidityTime(glite_catalog_ctx *ctx, const char *lfn,
	uint64_t time);

/**
 * \brief Bulk operation to update the validityTime of many files.
 *
 * The same time-value will be used for each files.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param lfns Array of files to be updated.
 * @param time The new time-value.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_updateValidityTime_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const lfns[], uint64_t time);

/**
 * \brief Read the directory given in the path argument, restricted to the pattern given in the second argument.
 *
 * The second argument may be null, in which case all the directory is listed.
 *
 * @param ctx The general catalog-context.
 * @param path The path of the directory.
 * @param pattern The pattern to match. May be null.
 * @param limit Maximal length of the result array.
 * @param offset Skip the first offset entries and start listing with entry number offset+1. 
 * @param withPermissions If withPermissions is not set then the FCEntries will not have a
 *        Permission object included, which speeds up listing considerably.
 * @param resultCount The number of entries in the return array. Never greater than limit.
 *
 * @return An array of FileCatalog Entries, limited by the limit parameter.
 */
glite_catalog_FCEntry **glite_fireman_readDir(glite_catalog_ctx *ctx,
	const char *path, const char *pattern, int limit, int64_t offset,
	int withPermissions, int *resultCount);

/**
 * \brief Returns list of matching LFNs from all subdirectories of the path given in the first argument.
 *
 * This method does not resolve * symlinks.
 * This method doesn not have offset, because traversal of a directory tree is
 * ambiquous, therefor the order of the returned entries cannot defined.
 *
 * @param ctx The general catalog-context.
 * @param startDir The starting directory.
 * @param pattern The pattern to match.
 * @param limit Maximal size of the resulting array.
 * @param resultCount [OUT] Size of the resutling array.
 *
 * @return An array of FileCatalog Entries containing the matching files.
 */
char **glite_fireman_locate(glite_catalog_ctx *ctx, const char *startDir,
	const char *pattern, int limit, int *resultCount);

/**
 * \brief Retrieves the GUID for given LFN.
 *
 * Both directories and files have guids.
 *
 * @param ctx The general catalog-context.
 * @param lfn The LFN we want the GUID for. 
 *
 * @return The GUID of the LFN.
 */
char *glite_fireman_getGuidForLfn(glite_catalog_ctx *ctx, const char *lfn);

/**
 * Retrieves list of GUIDs for given LFNs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param lfns Array of LFNs we want the GUIDs for.
 *
 * @return The array of GUIDs, same sized as the input array of LFNs.
 */
char **glite_fireman_getGuidForLfn_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const lfns[]);

/**
 * Retrieves the LFN for the given GUID.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID we want the LFN for.
 *
 * @return The corresponding LFN.
 */
char *glite_fireman_getLfnForGuid(glite_catalog_ctx *ctx, const char *guid);

/**
 * Retrieves the LFNs for the given GUIDs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param guids Array of GUIDs which we want the LFNs for.
 *
 * @return The array of the corresponding LFNs. The length of the output array
 *         is the same as that of the input array.
 */
char **glite_fireman_getLfnForGuid_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const guids[]);

/**
 * Get the detailed metadata about an LFN.
 *
 * @param ctx The general catalog-context.
 * @param lfn The LFN about which the details we want.
 *
 * @return The FileCatalog Entry with all the metadata.
 */
glite_catalog_FCEntry *glite_fireman_getFileCatalogEntry(glite_catalog_ctx *ctx,
	const char *lfn);

/**
 * Get detailed metadata about LFNs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param lfns Array of LFNs we want to query.
 *
 * @return An array of FileCatalog Entries. The length is the same as that of the input array.
 */
glite_catalog_FCEntry **glite_fireman_getFileCatalogEntry_multi(glite_catalog_ctx *ctx,
	int nitems, const char * const lfns[]);

/**
 * @}
 */

/**
 * \defgroup catalog_service_meta org.glite.data.catalog.service.meta.MetadataBase
 * @{
 */

/**
 * Sets the attributes of the given object. 
 *
 * @param ctx The general catalog-context.
 * @param item The string that uniquely identifies the entry in the 
 * catalog. It can be a GUID, an LFN, or any other type of identifier.
 *
 * @param nattributes Number of elements in the attributes array.
 * @param attributes Array of attribute objects.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_setAttributes(glite_catalog_ctx *ctx, const char *item,
	int nattributes, const glite_catalog_Attribute * const attributes[]);

/**
 * Clears the values for a group of attributes.
 *
 * @param ctx The general catalog-context.
 * @param item The item of which the attributes need to be cleared.
 * @param nattributes Number of elements in the attributes array.
 * @param attributes Array of the attribute names that are to be cleared.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_clearAttributes(glite_catalog_ctx *ctx, const char *item,
	int nattributes, const char * const attributes[]);

/**
 * Get the attributes of an entry in the catalog.
 *
 * @param ctx The general catalog-context.
 * @param item The string that uniquely identifies the entry in the
 * catalog. It can be a GUID, an LFN, or any other type of identifier.
 *
 * @param nattributes Number of elements in the attributes array.
 * @param attributes Array of the attribute names of which the values we are interested in.
 * @param resultCount [OUT] The length of the result array.
 *
 * @return An array containing the requested attribute names and values,
 *         and it may contain the type value, too. NULL, if there is no
 *         attribute with the given names in the catalog.
 */
glite_catalog_Attribute **glite_fireman_getAttributes(glite_catalog_ctx *ctx,
	const char *item, int nattributes, const char * const attributes[],
	int *resultCount);

/**
 * List all the attributes associated with the given entry.
 *
 * @param ctx The general catalog-context.
 * @param item  The string that uniquely identifies the entry in the
 * catalog. It can be a GUID, an LFN, or any other type of identifier.
 * @param resultCount [OUT] The length of the result array.
 *
 * @return An array of attribute objects with all the attributes associated with
 *         the entry, even those that do not have a value.
 */
glite_catalog_Attribute **glite_fireman_listAttributes(glite_catalog_ctx *ctx,
	const char *item, int *resultCount);

/**
 * \brief Performs a generic query on the catalog, returning the
 * corresponding item's identifiers.
 *
 * The query is simply a string that has to be understood by the service. For
 * example, if the catalog has a backend talking SQL, then this string
 * should be a valid SQL query.
 *
 * @param ctx The general catalog-context.
 * @param query The query string.
 * @param type 
 * @param limit The number of records to be returned in a single request.
 * @param offset The number of records that should be ignored from the
 *               beginning of the whole resultset of the query
 * @param resultCount [OUT] The length of the result array.
 *
 * @return The identifiers of all items corresponding to the given query. These
 *         identifiers are strings, each being a GUID, an LFN, or any other
 *         unique string identifier. 
 */
char **glite_fireman_query(glite_catalog_ctx *ctx, const char *query,
	const char *type, int limit, int offset, int *resultCount);

/**********************************************************************
 * Function prototypes - org.glite.data.catalog.service.replica.ReplicaCatalog
 */

/**
 * \brief Creates a set of new mappings in the Replica catalog.
 *
 * The RCEntry object contains the GUID, its stat and a list of SURLEntry 
 * objects giving all information about the SURLs associated with the given GUID.
 * Permissions are set in the GUIDStat object of the entry. If they are kept
 * empty, defaults are taken.
 *
 * @param ctx The general catalog-context.
 * @param entry The entry containing all the information of the SURL.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_createGuid(glite_catalog_ctx *ctx,
	const glite_catalog_RCEntry *entry);

/**
 * Bulk operation to create multiple entries. @see glite_fireman_createGuid()
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param entries Array of entries to be created.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_createGuid_multi(glite_catalog_ctx *ctx, int nitems,
	const glite_catalog_RCEntry * const entries[]);

/**
 * Adds a list of new SURLs for existing entry in the catalog, keyed on the GUID.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID that we add the elements to. 
 * @param nsurls Number of SURLs in the array. 
 * @param surls Array of SURLs to attach to the GUID.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_addGuidReplica(glite_catalog_ctx *ctx, const char *guid,
	int nsurls, const glite_catalog_SURLEntry * const surls[]);

/**
 * Update the stats of a GUID.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID to update.
 * @param stat The new GUIDStat object.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_updateGuidStat(glite_catalog_ctx *ctx, const char *guid,
	const glite_catalog_GUIDStat *stat);

/**
 * Bulk operation to update the stats of many GUIDs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in both arrays.
 * @param guids Array of GUIDs to be set.
 * @param stats Array of GUIDStat objects to set.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_updateGuidStat_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const guids[],
	const glite_catalog_GUIDStat * const stats[]);

/**
 * Sets the SURLs of a given GUID.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID on which we want to set the stats.
 * @param nsurls Array of SURLs in the array.
 * @param surls Array of SURLs to be set.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_updateSurlStat(glite_catalog_ctx *ctx, const char *guid,
	int nsurls, const glite_catalog_SURLEntry * const surls[]);

/**
 * \brief Let a specific SURL be the Master Replica of a GUID.
 *
 * There can be only one Master Replica in the catalog for each GUID.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID, which willl have the MR.
 * @param surl The SURL of the MR.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_setMasterReplica(glite_catalog_ctx *ctx, const char *guid,
	const char *surl);

/**
 * Bulk operation to set the MasterReplica of multiple GUIDs at once.
 * @see glite_fireman_setMasterReplica()
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param items Array of string-pairs, in each pair the first string is the GUID,
 *              the second one is the SURL to be set as MasterReplica.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_setMasterReplica_multi(glite_catalog_ctx *ctx, int nitems,
	const char *items[][2]);

/**
 * \brief Update the status bits of a GUID.
 *
 * Updates only those bits that are 1 in the mask, and sets them to
 * the new value specified in status.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID of which the status is to be updated.
 * @param status The new status bits.
 * @param mask The bitmask of the operation.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_updateStatus(glite_catalog_ctx *ctx, const char *guid,
	int32_t status, int32_t mask);

/**
 * \brief Bulk operation to set the status bits of many GUIDs.
 *
 * Sets the same status on every GUIDs. @see glite_fireman_updateStatus()
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param guids Array of GUIDs to update.
 * @param status Status bits to set.
 * @param mask The bitmask of the operation.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_updateStatus_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const guids[], int32_t status, int32_t mask);

/**
 * Remove a GUID-SURL mapping.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID.
 * @param surl The SURL.
 * @param removeGuidOnLastReplica Whether to remove the GUID too, if we remove the last replica.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_removeGuidReplica(glite_catalog_ctx *ctx, const char *guid,
	const char *surl, int removeGuidOnLastReplica);

/**
 * Bulk operation to remove multiple GUID-SURL mappings.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param items Array of string-pairs, in every pair the first
 *              string is the GUID, the second one is the SURL.
 * @param removeGuidOnLastReplica 
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_removeGuidReplica_multi(glite_catalog_ctx *ctx, int nitems,
	const char *items[][2], int removeGuidOnLastReplica);

/**
 * Remove a GUID with all existing mappings from the catalog.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID to be removed.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_removeGuid(glite_catalog_ctx *ctx, const char *guid);

/**
 * Removes multiple GUIDs will all their mappings from the catalog.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param guids Array of GUIDs to be removed.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_removeGuid_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const guids[]);

/**
 * \brief Sets default permissions for user. 
 *
 * Defaults to be used for newly created GUIDs where the GUIDStat object does
 * not contain any permissions. If the user does not have default user
 * permissions set, the global permissions are used.
 * Sets permission policy for a user used while creating new RCEntries with
 * no permissions specified. @see glite_fireman_setDefaultGlobalPermission()
 *
 * @param ctx The general catalog-context.
 * @param principal The name of this principal.
 * @param permission The permission object to be used.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_setDefaultPrincipalPermission(glite_catalog_ctx *ctx,
	const char *principal, const glite_catalog_Permission *permission);

/**
 * \brief Sets the global default permissions.
 *
 * If a user does not have its own default permissions, these global
 * permissions are used for new entries if no permission is specified.
 *
 * @param ctx The general catalog-context.
 * @param permission The permission object to be used.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_setDefaultGlobalPermission(glite_catalog_ctx *ctx,
	const glite_catalog_Permission *permission);

/**
 * Gets the GUID the SURL is mapped to.
 *
 * @param ctx The general catalog-context.
 * @param surl The SURL of which the GUID is needed.
 *
 * @return The GUID of the given SURL.
 */
char *glite_fireman_getGuidForSurl(glite_catalog_ctx *ctx, const char *surl);

/**
 * Bulk operation to get the GUIDs of many SURLs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param surls Array of SURLs to get their GUIDs.
 *
 * @return The array of GUIDs, the length is the same as that of the input array.
 */
char **glite_fireman_getGuidForSurl_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const surls[]);

/**
 * Gets the Master Replica of a GUID.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID which we want the MasterReplica of.
 *
 * @return The SURL of the MasterReplica.
 */
char *glite_fireman_getMasterReplica(glite_catalog_ctx *ctx, const char *guid);

/**
 * Bulk operation to get the MasterReplicas of many GUIDs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param guids Array of GUIDs which we want the MasterReplicas of.
 *
 * @return An array of SURLs of the MasterReplicas of the requested GUIDs.
 *         The array has the same length as the input array.
 */
char **glite_fireman_getMasterReplica_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const guids[]);

/**
 * \brief Lists GUID-SURL string pairs of the requested GUID.
 *
 * If a GUID has more SURLs, it is repeated in the list for its every SURL.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID which we need the SURLs of.
 * @param resultCount The length of the result array.
 *
 * @return An array of string-pairs interpreted as GUID-SURL pairs.
 */
char **glite_fireman_listSurlsByGuid(glite_catalog_ctx *ctx,
	const char *guid, int *resultCount);

/**
 * \brief Lists GUID-SURL string pairs of the requested GUIDs.
 *
 * If a GUID has more SURLs, it is repeated in the list for its every SURL.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param guids Array of GUIDs to be queried.
 * @param resultCount The length of the result array.
 *
 * @return An array of string-pairs interpreted as GUID-SURL pairs.
 */
glite_catalog_StringPairArray glite_fireman_listSurlsByGuid_multi(glite_catalog_ctx *ctx,
	int nitems, const char * const guids[], int *resultCount);

/**
 * Lists only the <b>valid</b> replicas of a GUID.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID which we want the valid replicas of.
 * @param withPermissions Whether we want permissions, or not. Without
 *                        permissions the request completes much quicker.
 *
 * @return The RCEntry of the GUID with all valid replicas.
 */
glite_catalog_RCEntry *glite_fireman_listReplicasByGuid(glite_catalog_ctx *ctx,
	const char *guid, int withPermissions);

/**
 * Bulk operation to get <b>valid</b> replicas of many GUIDs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param guids Array of GUIDs which we want the valid replicas of.
 * @param withPermissions Whether we want permissions, or not. Without
 *                        permissions the request completes much quicker.
 *
 * @return An array of RCEntries with all valid replicas. Every GUID
 *         has exactly one RCEntry in the return array.
 */
glite_catalog_RCEntry **glite_fireman_listReplicasByGuid_multi(glite_catalog_ctx *ctx,
	int nitems, const char * const guids[], int withPermissions);

/**
 * Convenience method to check the existence of a GUID in the catalog.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID we want to check.
 *
 * @return 0 when the GUID exists, -1 if it does not or there was an error.
 *         You get a ::NotExistsException in case the GUID does not exist.
 */
int glite_fireman_hasGuid(glite_catalog_ctx *ctx, const char *guid);

/**
 * Bulk operation to check the existence of multiple GUIDs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param guids Array of GUIDs to be checked.
 *
 * @return 0 when all the GUIDs exists, -1 if at least one does not or there was an error.
 *         You get a ::NotExistsException in case a GUID does not exist.
 */
int *glite_fireman_hasGuid_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const guids[]);

/**
 * Get the GUIDStat objects for given GUID.
 *
 * @param ctx The general catalog-context.
 * @param guid The GUID we want the stat for.
 *
 * @return The GUIDStat object.
 */
glite_catalog_GUIDStat *glite_fireman_getGuidStat(glite_catalog_ctx *ctx,
	const char *guid);

/**
 * Gets list of GUIDStat objects for given GUIDs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param guids Array of GUIDs we want the stats for.
 *
 * @return Array of GUIDStat objects. One element for each given GUID.
 */
glite_catalog_GUIDStat **glite_fireman_getGuidStat_multi(glite_catalog_ctx *ctx,
	int nitems, const char * const guids[]);

/**
 * Get the SURLStat object for a given SURL.
 *
 * @param ctx The general catalog-context.
 * @param surl The SURL we want the stats for.
 *
 * @return The corresponding SURLStat object.
 */
glite_catalog_SURLEntry *glite_fireman_getSurlStat(glite_catalog_ctx *ctx,
	const char *surl);

/**
 * Bulk operation to get the SURLStat objects of multiple SURLs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param surls Array of SURLs we want the stats of.
 * @param resultCount [OUT] The length of the return array.
 *
 * @return An array of SURLStat objects, one for each input SURL.
 */
glite_catalog_SURLEntry **glite_fireman_getSurlStat_multi(glite_catalog_ctx *ctx,
	int nitems, const char * const surls[], int *resultCount);

/**
 * \brief Get the default permissions for user.
 *
 * Defaults to be used for newly created
 * GUIDs where the GUIDStat object does not contain any permissions. If the user
 * does not have is default user permissions set, the global permissions are used.
 * If the user cannot be detected, default global permissions are used.
 *
 * @param ctx The general catalog-context.
 * @param principal The name of the principal for which we want the default permission.
 *
 * @return The permission object that defaults to the given principal.
 */
glite_catalog_Permission *glite_fireman_getDefaultPrincipalPermission(glite_catalog_ctx *ctx,
	const char *principal);

/**
 * \brief Gets the global default permissions.
 *
 * If a user does not have its own default permissions, these global permissions
 * are used for new entries if no permission is specified.
 *
 * @param ctx The general catalog-context.
 *
 * @return The default permission object of the catalog.
 */
glite_catalog_Permission *glite_fireman_getDefaultGlobalPermission(glite_catalog_ctx *ctx);

/**
 * @}
 */


/**
 * \defgroup catalog_service_fireman org.glite.data.catalog.service.composite.FiremanCatalog
 * @{
 */

/**
 * Creates entries based on LFN and SURL only. If GUID is not provided, it is
 * created by the catalog. Only file items are permitted here.
 *
 * @param ctx The general catalog-context.
 * @param entry The File and Replica Catalog Entry to be created.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_create(glite_catalog_ctx *ctx,
	const glite_catalog_FRCEntry *entry);

/**
 * Bulk operation to create entries based on LFN and SURL only.
 * If GUID is not provided, it is created by the catalog. Only file items are permitted here.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param entries Array of File and Replica Catalog Entries to be created.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_create_multi(glite_catalog_ctx *ctx, int nitems,
	const glite_catalog_FRCEntry * const entries[]);

/**
 * List all known SURLs registered to a given LFN.
 *
 * @param ctx The general catalog-context.
 * @param lfn The LFN we want the SURLs of.
 * @param withPermissions Wheter we want the permission block filled in.
 *
 * @return The FRCEntry with all valid SURLs.
 */
glite_catalog_FRCEntry *glite_fireman_listReplicas(glite_catalog_ctx *ctx,
	const char *lfn, int withPermissions);

/**
 * Bulk operation to list all known SURLs registered to given LFNs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param lfns Array of LFNs we want the SURLs of.
 * @param withPermission Wheter we want the permission block filled in.
 *
 * @return An array on FRCEntries, one for each given LFN.
 */
glite_catalog_FRCEntry **glite_fireman_listReplicas_multi(glite_catalog_ctx *ctx,
	int nitems, const char * const lfns[], int withPermission);

/**
 * Get the LFN for a SURL.
 *
 * @param ctx The general catalog-context.
 * @param surl The SURL which we want the LFN for.
 *
 * @return The LFN of the given SURL.
 */
char *glite_fireman_listLfn(glite_catalog_ctx *ctx, const char *surl);

/**
 * Bulk operation to get the LFN of multiple SURLs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param surls Array of SURLs which we want the LFNs for.
 *
 * @return An array of LFNs, one for each given SURL.
 */
char **glite_fireman_listLfn_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const surls[]);

/**
 * Get the Stat of a given LFN.
 *
 * @param ctx The general catalog-context.
 * @param lfn The LFN which we want the stats of.
 *
 * @return The corresponding Stat object.
 */
glite_catalog_Stat *glite_fireman_getStat(glite_catalog_ctx *ctx,
	const char *lfn);

/**
 * Get the Statof multiple LFNs.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param lfns Array of LFNs that we want the stats of.
 *
 * @return An array of Stat objects, one for each given LFN.
 */
glite_catalog_Stat **glite_fireman_getStat_multi(glite_catalog_ctx *ctx,
	int nitems, const char * const lfns[]);

/**
 * Get the LFNStat of a given LFN. More verbose than glite_fireman_getStat().
 *
 * @param ctx The general catalog-context.
 * @param lfn The LFN which we want the stats of.
 *
 * @return The corresponding LFNStat object.
 */
glite_catalog_LFNStat *glite_fireman_getLFNStat(glite_catalog_ctx *ctx,
	const char *lfn);

/**
 * Get the LFNStat of multiple LFNs. More verbose than glite_fireman_getStat_multi().
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param lfns Array of LFNs that we want the stats of.
 *
 * @return An array of LFNStat objects, one for each given LFN.
 */
glite_catalog_LFNStat **glite_fireman_getLFNStat_multi(glite_catalog_ctx *ctx,
	int nitems, const char * const lfns[]);

/**
 * New mapping creation for LFN-SURL pairs.
 *
 * @param ctx The general catalog-context.
 * @param lfn The LFN we want to add the replicas to.
 * @param nsurls Number of items in the array.
 * @param newSurls Array of SURLEntries to be mapped to the LFN.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_addReplica(glite_catalog_ctx *ctx, const char *lfn,
	int nsurls, const glite_catalog_SURLEntry * const newSurls[]);

/**
 * Removes LFN and all linked data (corresponding guid and replicas) from catalog.
 *
 * @param ctx The general catalog-context.
 * @param lfn The LFN to be removed.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_remove(glite_catalog_ctx *ctx, const char *lfn);

/**
 * Removes LFNs and all linked data (corresponding guid and replicas) from catalog.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param lfns Array of LFNs to be removed.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_remove_multi(glite_catalog_ctx *ctx, int nitems,
	const char * const lfns[]);

/**
 * Removes an LFN-SURL mapping.
 *
 * @param ctx The general catalog-context.
 * @param lfn The LFN.
 * @param surl The SURL.
 * @param removeLfnOnLastReplica Wheter to remove the LFN if we just removed the last
 *                               replica of it.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_removeReplica(glite_catalog_ctx *ctx, const char *lfn,
	const char *surl, int removeLfnOnLastReplica);

/**
 * Bulk operation to remove multiple LFN-SURL mappings.
 *
 * @param ctx The general catalog-context.
 * @param nitems Number of items in the array.
 * @param items Array of string-pairs to be unmapped, the first string is the LFN, the second one is the GUID.
 * @param removeLfnOnLastReplica Wheter to remove the LFN if we just removed the last
 *                               replica of it.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_removeReplica_multi(glite_catalog_ctx *ctx, int nitems,
	const char *items[][2], int removeLfnOnLastReplica);

/**
 * \brief Associates an existing directory with an existing schema.
 *
 * The semantics of this operation should be that all files inside the given
 * directory inherit and share this schema.
 *
 * @param ctx The general catalog-context.
 * @param directoryLfn The LFN of the directory.
 * @param schemaName The name of the schema to apply on the directory.
 *
 * @return 0 when successful and -1 in case of an error.
 */
int glite_fireman_associateDirWithSchema(glite_catalog_ctx *ctx,
	const char *directoryLfn, const char *schemaName);

/**
 * @}
 */


#ifdef __cplusplus
}
#endif

#endif /* GLITE_DATA_CATALOG_FIREMAN_SIMPLE_H */
